﻿using System;
using System.Text;
using System.IO;
using System.Linq;
using System.Collections.Generic;
using System.Text.RegularExpressions;

/// @author vesal
/// @version 18.10.2018
/// <summary>
/// Suodatetaan Korpin varausraportista näkyviin vain viikonloput ja iltavaraukset
/// Varaukset ovat tekstitiedostossa, jonka yksi rivi on muotoa:
/// <example>
///   Ag Auditorio 1 KTTP211 Taloustieteen peruskurssi KTTP2110- 216000 Ma 22.10        10:15-12:00             43    Mangeloja Esa                
/// </example>
/// Lisäksi poistetaan sellaiset varaukset, jotka vastaavat tiedostossa pois.txt olevaan regexpl ehtoon.
/// Pois.txt voi olla esim
/// <example>
/// Auto:
/// Remontti.*
/// Huolto.*
/// ^R \(Ryhtilä\) .*
/// ^U2.*
/// ^PS \(Palloi
/// ^RS \(Rytmii
/// ^TS \(Teline
/// ^VS \(Voimiste/// </example>
/// </summary>
public class Suodata
{
    /// <summary>
    /// Luetaan vasraustiedosto ja poistettavien ehto ja tulostetaan jäljelle jääneet
    /// </summary>
    public static void Main(string[] args)
    {
        string tiedostonNimi = "reportOrgShow.txt";
        if (args.Length >= 1) tiedostonNimi = args[0];
        string[] rivit = File.ReadAllLines(tiedostonNimi);
        string poisNimi = "pois.txt";
        if (args.Length >= 2) poisNimi = args[1];
        string[] poistettavat = { "Auto:", "Remontti   ", "Huolto   ", @"R \(Ryhtilä\) ", "^U2   " };
        try
        {
            poistettavat = File.ReadAllLines(poisNimi);
        } catch (FileNotFoundException) { 
        }


        List<string> tulos = new List<string>();
        List<string> poistetut = new List<string>();

        foreach (string rivi in rivit)
        {
            if ( OnkoViikonloppu(rivi) || OnkoIlta(rivi) )
                if (OnkoPoistettava(rivi, poistettavat))
                    poistetut.Add(rivi);
                else tulos.Add(rivi);
        }

        File.WriteAllLines("lista.txt", tulos, Encoding.UTF8);
        File.WriteAllLines("poistetut.txt", poistetut, Encoding.UTF8);
    }


    /// <summary>
    /// Tutkitaan täyttääkö rivi jonkin poistamisehdon
    /// </summary>
    /// <param name="rivi">tutkittava rivi</param>
    /// <param name="poistettavat">taulukko regexp ehtoja joista jonkun toteutuessa poistetaan</param>
    /// <returns>true jos poistettava</returns>
    /// <example>
    /// <pre name="test">
    /// string[] poistettavat = { "Auto:", "Remontti   ", "Huolto   ", @"^R \(Ryhtilä\) ", "^U2   " };
    /// OnkoPoistettava("Henkilöauto: Mosse", poistettavat) === false;
    /// OnkoPoistettava("HenkilöAuto: Mosse", poistettavat) === true;
    /// OnkoPoistettava("Remonttireiska", poistettavat) === false;
    /// OnkoPoistettava("Remontti     reiska", poistettavat) === true;
    /// OnkoPoistettava(" R (Ryhtilä) ", poistettavat) === false;
    /// OnkoPoistettava("R (Ryhtilä) ", poistettavat) === true;
    /// </pre>
    /// </example>
    public static bool OnkoPoistettava(string rivi, string[] poistettavat)
    {
        foreach (string pois in poistettavat)
        {
            if (pois.Trim() == "") continue;
            Regex rx = new Regex(pois);
            if (rx.Matches(rivi).Count > 0) return true;
        }
        return false;
    }


    private static Regex rx = new Regex(@".*([0-9][0-9]):([0-9][0-9])-([0-9][0-9]):([0-9][0-9]).*", RegexOptions.Compiled);

    /// <summary>
    /// Tutkii onko rivi sellainen että se on iltajanvaraus
    /// </summary>
    /// <param name="rivi">tutkittava rivi</param>
    /// <returns>true jos iltavaraus</returns>
    /// <example>
    /// <pre name="test">
    ///   OnkoIlta("Ag Alfa Ma 22.10 08:00-12:00   43") === false
    ///   OnkoIlta("Ag Alfa Ma 22.10 16:00-18:00   43") === false
    ///   OnkoIlta("Ag Alfa Ma 22.10 08:00-19:00   43") === true
    ///   OnkoIlta("Ag Alfa Ma 22.10 18:15-19:00   43") === true
    ///   OnkoIlta("") === false
    /// </pre>
    /// </example>
    public static bool OnkoIlta(string rivi)
    {
        MatchCollection matches = rx.Matches(rivi);
        if (matches.Count < 1) return false;
        double loppuklo = Double.Parse(matches[0].Groups[3].Value) + Double.Parse(matches[0].Groups[4].Value) / 60;
        return loppuklo > 18;
    }


    /// <summary>
    /// Tutkii onko rivi viikonloppuvaraus
    /// </summary>
    /// <param name="rivi">tutkittava rivi</param>
    /// <returns>true jos viikonlopun varaus</returns>
    /// <example>
    /// <pre name="test">
    ///   OnkoViikonloppu("Ag Alfa La 20.10 08:00-18:00") === true
    ///   OnkoViikonloppu("Ag Alfa Su 21.10 08:00-18:00") === true
    ///   OnkoViikonloppu("Ag Alfa Ma 22.10 08:00-12:00") === false
    ///   OnkoViikonloppu("") === false
    /// </pre>
    /// </example>
    public static bool OnkoViikonloppu(string rivi)
    {
        string[] paivat = { " La ", " Su " };
        foreach (string pv in paivat)
            if (rivi.IndexOf(pv) >= 0) return true;
        return false;
    }
}
