package utilities;


/// <summary>
/// Tekee "satunnaislukugeneraattorin joka muodostaa lukuja LGC-sarjasta
/// ks: http://en.wikipedia.org/wiki/Linear_congruential_generator
/// <pre>
/// s(n+1)= ( s(n) * a + c ) mod m
/// </pre>
/// </summary>
/// Tm itse tehty generaattori on siksi, ett voidaan varmasti muodostaan
//tietynlaisia,
/// aina samanlaisia, lukuja erilaisia ajoitustestej varten.
public class RandomGen {
	private int c;
	private int a;
	private int sn;
	private int m;
	private static int A = 1664525;
	private static int C = 1013904223;

	// / <summary>
	// / Alustetaan generaattori
	// / </summary>
	// / <param name="m">montaako eri arvoa tuotetaan</param>
	// / <param name="sn">siemenluku</param>
	// / <param name="a">kerroin</param>
	// / <param name="c">lisystermi</param>
	// / <example>
	// / <pre name="test">
	// / RandomGen rand = new RandomGen();
	// / rand.Next() === 1013904223;
	// / rand.Next() === 1196435762;
	// / rand.Next() === 1372387049;
	// / </pre>
	// / </example>
	public RandomGen(int m, int sn, int a, int c) {
		this.a = a;
		this.c = c;
		this.sn = sn;
		this.m = m;
		if (m == 0)
			this.m = 0x7fffffff;
	}

	public RandomGen(int m, int sn, int a) {
		this(m, sn, a, C);
	}

	public RandomGen(int m, int sn) {
		this(m, sn, A, C);
	}

	public RandomGen(int m) {
		this(m, 0, A, C);
	}

	public RandomGen() {
		this(0, 0, A, C);
	}

	// / <summary>
	// / Alustetaan siemenluku valituksi.
	// / </summary>
	// / <param name="sn"></param>
	// / <example>
	// / <pre name="test">
	// / RandomGen rand = new RandomGen(10);
	// / rand.Next() === 3;
	// / rand.Next() === 2;
	// / rand.Set(0);
	// / rand.Next() === 3;
	// / rand.Next() === 2;
	// / </pre>
	// / </example>
	public void Set(int sn) {
		this.sn = sn;
	}

	// / <summary>
	// / Palautetaan seuraava pseudosatunnaisluku
	// / </summary>
	// / <returns></returns>
	// / <pre name="test">
	// / RandomGen rand = new RandomGen(10);
	// / rand.Next() === 3;
	// / rand.Next() === 2;
	// / rand.Next() === 9;
	// / rand.Next() === 6;
	// / rand.Next() === 7;
	// /
	// / rand = new RandomGen(10,-1,1,1);
	// / rand.Next() === 0;
	// / rand.Next() === 1;
	// / rand.Next() === 2;
	// / rand.Next() === 3;
	// / rand.Next() === 4;
	// /
	// / rand = new RandomGen(10,-2,1,2);
	// / rand.Next() === 0;
	// / rand.Next() === 2;
	// / rand.Next() === 4;
	// / rand.Next() === 6;
	// /
	// / rand = new RandomGen(1000,1,2,0);
	// / rand.Next() === 2;
	// / rand.Next() === 4;
	// / rand.Next() === 8;
	// / rand.Next() === 16;
	// / </pre>
	// / </example>
	public int Next() {
		int snp1 = a * sn + c;
		sn = snp1;
		return (snp1 & 0x7fffffff) % m;
	}

	// / <summary>
	// / Alustaa taulukon joukolla satunnaisia lukuja [0,m[
	// / </summary>
	// / <param name="n">alkioiden lukumr</param>
	// / <param name="m">suurin tuotettava alkio</param>
	// / <param name="sn">siemineluku</param>
	// / <param name="a">generaattorin kerroin</param>
	// / <param name="c">generaattorin lisystermi</param>
	// / <returns>tytetty taulukko</returns>
	// / <example>
	// / <pre name="test">
	// / string.Join(" ",RandomGen.Tayta(5,10)) === "3 2 9 6 7";
	// / </pre>
	// / </example>
	public static int[] Tayta(int n, int m, int sn, int a, int c) {
		int[] tulos = new int[n];
		RandomGen rand = new RandomGen(m, sn, a, c);
		for (int i = 0; i < n; i++)
			tulos[i] = rand.Next();
		return tulos;
	}

	public static int[] Tayta(int n, int m, int sn, int a) {
		return Tayta(n, m, sn, a, C);
	}

	public static int[] Tayta(int n, int m, int sn) {
		return Tayta(n, m, sn, A, C);
	}

	public static int[] Tayta(int n, int m) {
		return Tayta(n, m, 0, A, C);
	}

	public static int[] Tayta(int n) {
		return Tayta(n, 0, 0, A, C);
	}

	public static int[] Tayta() {
		return Tayta(0, 0, 0, A, C);
	}

}
